/* libnfnetlink.h: Header file for the Netfilter Netlink library.
 *
 * Jay Schulist <jschlst@samba.org>, Copyright (c) 2001.
 *
 * This software may be used and distributed according to the terms
 * of the GNU General Public License, incorporated herein by reference.
 */

#ifndef __LIBNFNETLINK_H
#define __LIBNFNETLINK_H

#include <netinet/in.h>
#include <asm/types.h>
#include <linux/if.h>
#include <linux/netlink.h>
#include <linux/netfilter_ipv4/ip_conntrack.h>
#include <linux/netfilter_ipv4/ip_conntrack_tcp.h>
#include <linux/netfilter_ipv4/ip_conntrack_ftp.h>
#include <linux/netfilter_ipv4/ip_conntrack_icmp.h>
#include <linux/netfilter_ipv4/ip_conntrack_tuple.h>
#include <linux/nfnetlink.h>

#define NFNL_BUFFSIZE   8192

struct nfnl_handle {
        int                     fd;
        struct sockaddr_nl      local;
        struct sockaddr_nl      peer;
        u_int32_t               seq;
        u_int32_t               dump;
};

extern int nfnl_open(struct nfnl_handle *nfh, unsigned subscriptions);
extern int nfnl_close(struct nfnl_handle *nfh);
extern int nfnl_send(struct nfnl_handle *nfh, char *buf, int len);
extern int nfnl_listen(struct nfnl_handle *nfnl,
        int (*handler)(struct sockaddr_nl *, struct nlmsghdr *n, void *),
        void *jarg);
extern int nfnl_talk(struct nfnl_handle *nfnl, struct nlmsghdr *n, pid_t peer,
        unsigned groups, struct nlmsghdr *answer,
        int (*junk)(struct sockaddr_nl *, struct nlmsghdr *n, void *),
        void *jarg);
extern int nfnl_wilddump_request(struct nfnl_handle *nfh, int family, int type);extern int nfnl_dump_request(struct nfnl_handle *nfh, int type, void *req,
        int len);
extern int nfnl_dump_filter(struct nfnl_handle *nfh,
        int (*filter)(struct sockaddr_nl *, struct nlmsghdr *n, void *),
        void *arg1,
        int (*junk)(struct sockaddr_nl *,struct nlmsghdr *n, void *),
        void *arg2);
extern int nfnl_addattr32(struct nlmsghdr *n, int maxlen, int type,
        u_int32_t data);
extern int nfnl_addattr_l(struct nlmsghdr *n, int maxlen, int type, void *data,
        int alen);
extern int nfnl_cta_addattr32(struct nfattr *nfa, int maxlen, int type,
        u_int32_t data);
extern int nfnl_cta_addattr_l(struct nfattr *nfa, int maxlen, int type,
        void *data, int alen);
extern int nfnl_parse_attr(struct nfattr *tb[], int max, struct nfattr *nfa,
        int len);

#endif  /* __LIBNFNETLINK_H */
