/* nf.h: header file for nf.
 */

#ifndef __NF_H
#define __NF_H

#include <linux/prefetch.h>

struct nf_head {
	struct nf_head *next;
	struct nf_head *prev;
};

struct nf_entry {
	struct nf_head list;

        char table_name[IPT_TABLE_MAXNAMELEN];
        char chain_name[IPT_FUNCTION_MAXNAMELEN];
        unsigned int entry_num;

        struct ipt_ip ip;
        unsigned int nfcache;
        struct ipt_counters counters;

        /* XXX: The matches (if any), then the target. */
        unsigned char elems[0];
};

struct nf_chain {
	struct nf_head list;

	char name[IPT_FUNCTION_MAXNAMELEN];
	char policy[IPT_FUNCTION_MAXNAMELEN];
	struct ipt_counters counters;

        int entry_num;
        struct nf_head entries;
};

struct nf_table {
	struct nf_head list;

        char name[IPT_TABLE_MAXNAMELEN];

        int chain_num;
        struct nf_head chains;
};

#define nf_init_list_head(ptr) do { \
        (ptr)->next = (ptr); (ptr)->prev = (ptr); \
} while (0)

#define nf_list_entry(ptr, type, member) \
        ((type *)((char *)(ptr)-(unsigned long)(&((type *)0)->member)))

#define nf_list_for_each(pos, head) \
        for (pos = (head)->next, prefetch(pos->next); pos != (head); \
                pos = pos->next, prefetch(pos->next))

/* Return pointer to first true entry, if any, or NULL.  A macro
   required to allow inlining of cmpfn. */
#define nf_list_find(head, cmpfn, type, args...)        \
({                                                      \
        const struct nf_head *__i = (head);           	\
                                                        \
        do {                                            \
                __i = __i->next;                        \
                if (__i == (head)) {                    \
                        __i = NULL;                     \
                        break;                          \
                }                                       \
        } while (!cmpfn((const type)__i , ## args));    \
        (type)__i;                                      \
})

extern void nf_list_del(struct nf_head *entry);
extern void nf_list_add(struct nf_head *new, struct nf_head *prev, 
	struct nf_head *next);
extern void nf_list_add_tail(struct nf_head *new, struct nf_head *head);

enum proto_defs {
	IPV4_CT_TCP = 0,
	IPV4_CT_UDP,
	IPV4_CT_ICMP
};

#define FMT_NUMERIC     0x0001
#define FMT_COUNTERS    0x0002
#define FMT_KILOMEGAGIGA 0x0004
#define FMT_NOTABLE     0x0010
#define FMT_NOTARGET    0x0020
#define FMT_VIA         0x0040
#define FMT_NONEWLINE   0x0080
#define FMT_LINENUMBERS 0x0100
#define FMT_VERBOSE	0x0200
#define FMT_NOOPTIONS	0x0400

#endif	/* _NF__H */
